<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

include_once(_PS_MODULE_DIR_.'retino/classes/WebserviceSpecificManagementRetinoOrder.php');

class Retino extends Module
{
    public function __construct()
    {
        $this->name = 'retino';
        $this->tab = 'others';
        $this->version = '0.0.1';
        $this->author = 'Retino';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '1.6',
            'max' => _PS_VERSION_
        ];
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = 'Retino';
        $this->description = $this->l('Retino integration for Prestashop >= 1.6');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    public function install()
    {
        return parent::install()
                && $this->registerHook('addWebserviceResources')
                && $this->setupModule();
    }

    public function uninstall()
    {
        return parent::uninstall()
                && $this->destroyModule();
    }

    protected function setupModule()
    {
        $result = true;

        // enable webservices
        Configuration::updateGlobalValue('PS_WEBSERVICE', true);

        // create api key
        $webservice_key = new WebserviceKey();
        $webservice_key->key = $this->generateKey();
        $webservice_key->description = 'Retino webservice key';
        $result &= $webservice_key->add();

        // I think that retino_* resources are not registered yet so this method does not work properly
        // WebserviceKey::setPermissionForAccount(
        //     (int) $webservice_key->id,
        //     array(
        //         'retino_orders' => array('GET' => true),
        //         'retino_changes' => array('GET' => true),
        //         'order_histories' => array('GET' => true),
        //         'orders' => array('GET' => true),
        //     )
        // );

        $resources = array(
            'retino_changes',
            'retino_order',

            'order_histories',
            'orders',
            'addresses',
            'customers',
            'deliveries',
            'order_invoices',
            'products',
            'states',
            'currencies',
            'countries',
            'languages',
            'carriers',
        );

        $sql = 'INSERT INTO `' . _DB_PREFIX_ . 'webservice_permission` (`id_webservice_permission` ,`resource` ,`method` ,`id_webservice_account`) VALUES ';
        foreach ($resources as $resource) {
            $sql .= '(NULL , \'' . pSQL($resource) . '\', \'GET\', ' . (int) $webservice_key->id . '), ';
        }
        $sql = rtrim($sql, ', ');
        $result &= Db::getInstance()->execute($sql);

        // Save module configuration, it will be displayed at the configuration page
        $this->setConfigValues(array(
            'webservicekey_id' => $webservice_key->id,
            'webservicekey_key' => $webservice_key->key,
            'last_successful_import_readable' => null,
            'last_successful_import_timestamp' => null,
        ));

        PrestaShopLogger::addLog('Retino module installed successfuly', 1, null, null, null, true);

        return $result;
    }

    public function destroyModule()
    {
        $config = $this->unsetConfigValues();

        if (array_key_exists('webservicekey_id', $config)) {
            $id = $config['webservicekey_id'];
            $webservice_key = new WebserviceKey($id);
            $webservice_key->delete();
        }

        PrestaShopLogger::addLog('Retino module uninstalled successfuly', 1, null, null, null, true);

        return true;
    }

    /**
     * Configuration page
     */
    public function getContent()
    {
        $configValues = $this->getConfigValues();
        $this->context->smarty->assign('api_key', $configValues['webservicekey_key']);

        if (array_key_exists('last_successful_import_readable', $configValues)) {
            $this->context->smarty->assign('last_successful_import', $configValues['last_successful_import_readable']);
        } else {
            $this->context->smarty->assign('last_successful_import', null);
        }

        $this->context->smarty->assign('eshop_name', $this->context->shop->name);
        $this->context->smarty->assign('eshop_url', $this->context->shop->getBaseURL(true));
        $this->context->smarty->assign('email', $this->context->employee->email);
        $this->context->smarty->assign('firstname', $this->context->employee->firstname);
        $this->context->smarty->assign('lastname', $this->context->employee->lastname);
        $this->context->smarty->assign('prestashop_version', _PS_VERSION_);

        return $this->display(__FILE__, 'views/templates/admin/configuration.tpl');
    }

    /**
     * Get configuration array from database
     * @return array
     */
    public function getConfigValues()
    {
        return json_decode(Configuration::get($this->name), true);
    }

    /**
     * Set configuration array to database
     * @param array $config
     * @param bool $merge when true, $config can be only a subset to modify or add additional fields
     * @return array
     */
    public function setConfigValues($config, $merge = false)
    {
        if ($merge) {
            $config = array_merge($this->getConfigValues(), $config);
        }

        if (Configuration::updateValue($this->name, json_encode($config))) {
            return $config;
        }

        return false;
    }

    /**
     * Unset module configuration
     * @return array
     */
    public function unsetConfigValues()
    {
        if (Configuration::hasKey($this->name)) {
            $config = $this->getConfigValues();
            Configuration::deleteByName($this->name);
            return $config;
        }
    }

    public function hookAddWebserviceResources()
    {
        $added_resources['retino_order'] = [
            'description' => 'Retino order detail',
            'specific_management' => true,
        ];
        return $added_resources;
    }

    protected function generateKey($length = 32) {
        $characters = '123456789ABCDEFGHIJKLMNPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }
}